#!/usr/bin/env bash

set -o allexport
source "${PYTEST_ENVIRONMENT}"
set +o allexport

# NOTE: only set -x after sourcing secret values, otherwise they are leaked
set -u -x

TEST_RESULT=0


# wait as in registrar_prep.sh, not sure if necessary
kubectl exec deployment/my-helm-release-registrar -- ./wait-initialized.sh
kubectl exec deployment/my-helm-release-renderer -- ./wait-initialized.sh

# # preprocessor
cat preprocessed_list.csv | while read product ; do
    echo $product
    kubectl exec deployment/my-helm-release-preprocessor -- preprocessor preprocess --config-file /config.yaml "$product"
done

# registrar
jq -c '.[]' product_list.json | while read product_stac_json ; do
    echo $product_stac_json
    kubectl exec deployment/my-helm-release-registrar -- registrar --config-file /config.yaml register items "${product_stac_json}"
done

# copy registrar result here to inspect it in test
kubectl cp default/`kubectl get po -l app.kubernetes.io/name=registrar -o name | cut -d'/' -f2 `:/mnt/ /mnt/registrar

# run tests (i.e. verification of output of above commands)

kubectl get all -n default
# expose database, renderer and client ports externally
kubectl port-forward svc/my-helm-release-database 5432:5432 &
# kubectl port-forward svc/my-helm-release-client 80:80 &
kubectl port-forward svc/my-helm-release-renderer 81:80 &

KUBECTL_PORT_FORWARD_PID=$!

pytest test_preprocessor.py test_registrar.py test_renderer.py
let "TEST_RESULT+=$?"

rm  -rf ../data/preprocessor_results/
sleep 5
# ingestor tests
kubectl cp ./tpzonlineitem.xml default/$(kubectl get po -l app.kubernetes.io/name=ingestor -o name | cut -d'/' -f2):/mnt/data
kubectl cp ./defected_report.xml default/$(kubectl get po -l app.kubernetes.io/name=ingestor -o name | cut -d'/' -f2):/mnt/data

sleep 40  # run ingestor
kubectl exec deployment/my-helm-release-ingestor -- ls /var/ingestor/success > success_list.csv
kubectl exec deployment/my-helm-release-ingestor -- ls /var/ingestor/fail > failure_list.csv

#client
#TODO

# fetch logs for ingestor tests
kubectl logs $(kubectl get po -l app.kubernetes.io/name=ingestor -o name | cut -d'/' -f2) > ingestor_log.txt
pytest test_ingestor.py
let "TEST_RESULT+=$?"

# copy registrar result here to inspect it in test
kubectl cp default/`kubectl get po -l app.kubernetes.io/name=registrar -o name | cut -d'/' -f2 `:/mnt/ /mnt/registrar

# pytest --fullpipeline test_renderer.py test_registrar.py
# let "TEST_RESULT+=$?"

rm  -rf ../data/preprocessor_results/

kill $KUBECTL_PORT_FORWARD_PID

exit $TEST_RESULT
