import os
import pytest
import csv
import re
import datetime

from osgeo import gdal


def list_dir_items(filename):
    output_list = []
    with open(filename, newline="") as csvfile:
        spamreader = csv.reader(csvfile, delimiter=" ", quotechar="|")
        for row in spamreader:
            for item in row:
                output_list.append(item)

    return output_list


def test_ingestion_reporting():
    success_list = list_dir_items("success_list.csv")
    failure_list = list_dir_items("failure_list.csv")
    assert "tpzonlineitem.xml" in success_list
    assert "defected_report.xml" in failure_list


def test_preprocessor(
    products=[["KS03_AIS_PSH_1G_20160101T113020_20160101T113022_KGS_019339_75CE.tar"]]
):
    products_list = []
    container = '../data/preprocessor_results'
    result_list = os.listdir(container)
    for product in products:
        products_list.append(product[0])

    # check if there are preprocessed results in the buckets
    assert len(result_list) > 0
    result_list.sort()
    products_list.sort()
    assert result_list == products_list

    preprocessed_list = []
    for item in products_list:
        files = os.listdir('%s/%s' % (container, item))
        for data in files:
            if data.endswith("tif"):
                preprocessed_list.append('%s/%s/%s' % (container, item, data))
    for item in preprocessed_list:
        image = gdal.Open(item, gdal.GA_ReadOnly)
        srcband = image.GetRasterBand(1)
        assert srcband.Checksum() is not None
        assert srcband.Checksum() > 0


def test_error_logging():
    with open("ingestor_log.txt", "r") as log_file:
        lines = log_file.read().splitlines()
        assert len(lines) > 0
        error_lines = list(filter(lambda x: "ERROR:" in x, lines))
        xml_error_lines = list(
            filter(lambda x: "lxml.etree.XMLSyntaxError" in x, lines)
        )
        log_file.close()

    for line in error_lines:
        # check the error notification
        assert "Failed to parse XML." in line
        match = re.search("[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}", line)
        date = datetime.datetime.strptime(match.group(), "%Y-%m-%d %H:%M:%S").date()
        # check the associated error time
        assert type(date) == datetime.date

    for line in xml_error_lines:
        # Check the specific expected error reprot
        error_msg = "Start tag expected, '<' not found, line 1, column 1"
        assert error_msg in line
