import os
import csv

import pytest
import psycopg2
from xml.etree import ElementTree

from pathlib import Path


@pytest.fixture(scope="session")
def connection():
    connect_args = dict(
        dbname=os.environ["DB_NAME"],
        user=os.environ["DB_USER"],
        password=os.environ["DB_PW"],
        host="localhost",  # works via kubectl portforward
        port=os.environ["DB_PORT"],
    )
    print("connection args")
    from pprint import pprint

    pprint(connect_args)

    with psycopg2.connect(**connect_args) as connection:
        yield connection


@pytest.fixture
def identifiers(fullpipeline):
    list_of_identifiers_csv = "./product_list.csv"
    if fullpipeline:
        list_of_identifiers_csv = "./product_list_full_integration.csv"
    with open(list_of_identifiers_csv) as f:
        yield csv.reader(f)


def query_eo_object(connection, eo_id):
    query = f"SELECT identifier FROM coverages_eoobject WHERE identifier = '{eo_id}';"
    with connection.cursor() as cursor:
        cursor.execute(query)
        return cursor.fetchone()[0]


def sanitize_path(path):
    invalid_windows_chars = '<>:"/\|?* '
    for char in invalid_windows_chars:
        path = path.replace(char, "_")
    return path


def test_db_name(connection, identifiers):
    for row in identifiers:
        identifier = row[0].split("/")[4]
        coverage_id = query_eo_object(connection, identifier)
        db_name = connection.get_dsn_parameters()["dbname"]

        assert coverage_id == identifier
        assert db_name == os.environ["DB_NAME"]


def compare_links(product_file, product, asset_name):

    content = product_file.read_text()
    root = ElementTree.fromstring(content.strip())

    urls = root.findall("{http://www.telespazio.com/CSCDA/CDD/PDAS}URL")
    wms_link = urls[1].find("{http://www.telespazio.com/CSCDA/CDD/PDAS}URL").text
    wcs_link = urls[0].find("{http://www.telespazio.com/CSCDA/CDD/PDAS}URL").text

    wms_capabilities = "https://panda.copernicus.eu/emg/ows?service=wms&request=GetCapabilities&amp&cql=identifier="
    wcs_capabilities = "https://panda.copernicus.eu/emg/ows?service=wcs&request=GetCapabilities&amp&cql=identifier="
    expected_wms_link = '%s"%s"' % (wms_capabilities, product)
    expected_wcs_link = '%s"%s_%s"' % (wcs_capabilities, product, asset_name)

    assert expected_wms_link.replace("&amp&", "&") == wms_link

    assert expected_wcs_link.replace("&amp&", "&") == wcs_link


def test_reporting(identifiers):
    report_list = list(Path("/mnt/registrar").iterdir())
    assert len(report_list) > 0

    for row in identifiers:
        report_found = False
        for report_file in report_list:
            identifier = row[0].split("/")[4]
            asset_name = row[1]
            if sanitize_path(identifier) in report_file.name:
                report_found = True
                compare_links(report_file, identifier, asset_name)
        assert report_found
